package com.example.gengmei_flutter_plugin

import android.Manifest
import android.annotation.TargetApi
import android.app.Activity
import android.app.Application
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.util.Log
import android.widget.Toast
import com.example.gengmei_flutter_plugin.utils.addTo
import com.example.myimagepicker.repository.ImageRespository
import io.flutter.plugin.common.*
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.reactivex.disposables.CompositeDisposable
import java.lang.invoke.MethodHandle
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class GengmeiFlutterPlugin : MethodCallHandler {

    val disposable = CompositeDisposable();
    var result: Result? = null;
    private var record: HashMap<String, ArrayList<HashMap<String, Any>>>? = null;

    companion object {
        const val IMAGE_PICKER = "scan_image_picker"
        const val PREMISSION = 10090
        lateinit var resign: Registrar;
        var listener: EventChannel.EventSink? = null
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            if (registrar.activity() == null) {
                return
            }
            resign = registrar;
            val gengmeiFlutterPlugin = GengmeiFlutterPlugin();
            val channel = MethodChannel(registrar.messenger(), "gengmei_flutter_plugin")
            channel.setMethodCallHandler(gengmeiFlutterPlugin)
            val eventChannel = EventChannel(registrar.messenger(), "gengmei_flutter_plugin_event")
            eventChannel.setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(p0: Any?, p1: EventChannel.EventSink?) {
                    listener = p1;
                }

                override fun onCancel(p0: Any?) {
                    print("ON CANCEL !!!   ");
                    listener = null;
                }
            })
        }
    }

    @TargetApi(Build.VERSION_CODES.M)
    private fun checkPermission() {
        val activity = resign.activity()
        val writePremission = activity.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
        val cameraPremission = activity.checkSelfPermission(Manifest.permission.CAMERA);
        if (writePremission != PackageManager.PERMISSION_GRANTED ||
                cameraPremission != PackageManager.PERMISSION_GRANTED) {
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission
                            .WRITE_EXTERNAL_STORAGE)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用！", Toast.LENGTH_SHORT).show()
            }
            if (activity.shouldShowRequestPermissionRationale(Manifest.permission.CAMERA)) {
                Toast.makeText(activity.applicationContext, "请开通相关权限，否则无法正常使用本应用", Toast.LENGTH_SHORT).show()
            }
            //申请权限
            activity.requestPermissions(arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE
                    , Manifest.permission.CAMERA), PREMISSION);
        } else {
            Toast.makeText(activity.applicationContext, "授权成功！", Toast.LENGTH_SHORT).show();
            ScanImage();
        }
        resign.addRequestPermissionsResultListener { id, permissions, grantResults ->
            if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                ScanImage()
            } else {
                Toast.makeText(activity.applicationContext, "请同意权限", Toast.LENGTH_SHORT).show()
                result?.run {
                    this.error("没有权限！！", "no premission", "没有权限！！");
                }
            }
            true;
        }
    }


    override fun onMethodCall(call: MethodCall, result: Result) {
        this.result = result
        if (call.method == IMAGE_PICKER) {
            checkPermission()
        } else {
            result.notImplemented()
        }
    }


    fun ScanImage() {
        ImageRespository.getInstance().scanPhoneImage(resign.context().applicationContext).subscribe(
                {
                    result?.run {
                        record = it;
                        this.success(it);
                    }
                    savePreview();
                }, {
            result?.run {
                this.error(it.message, it.message, it);
            }
        }).addTo(disposable);
    }

    fun savePreview() {
        ImageRespository.getInstance().savePreviewImg(resign.context().applicationContext, object : ImageRespository.savePreviewListener {
            override fun onSuccess(data: HashMap<String, ArrayList<HashMap<String, Any>>>) {
                Handler(resign.activeContext().applicationContext.mainLooper).post {
                    listener?.run {
                        this.success(data)
                    }
                }
            }
        });
    }
}
